@@ -1465,7 +1465,7 @@ AtPlotterWidget pw;
      double dummy_min, dummy_max;
      Boolean changed;
 
-#ifdef TRACE
+#ifdef NEVERTRACE
      fprintf(stderr,
 	     "Axis Values: WAS %.1f,%.1f %.1f,%.1f %.1f,%.1f %.1f,%.1f\n",
 	     bbp->xmin, bbp->xmax, bbp->ymin, bbp->ymax,
@@ -1482,6 +1482,10 @@ AtPlotterWidget pw;
       * get min/max set.
       */
      if (bbp->xmax > bbp->xmin) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 1\n");
+#endif
+
 	  if (pp->xaxis)
 	       AtAxisAskRange(pp->xaxis, &bbp->xmin, &bbp->xmax);
 	  else
@@ -1489,11 +1493,17 @@ AtPlotterWidget pw;
 			  "AtPlotter has no X axis defined");
      }
      else if (pp->xaxis && ISDISPLAYED(pp->xaxis)) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 2\n");
+#endif
 	  dummy_min = LIN_MIN;
 	  dummy_max = LIN_MAX;
 	  AtAxisAskRange(pp->xaxis, &dummy_min, &dummy_max);
      }
      if (bbp->ymax > bbp->ymin) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 3\n");
+#endif
 	  if (pp->yaxis)
 	       AtAxisAskRange(pp->yaxis, &bbp->ymin, &bbp->ymax);
 	  else
@@ -1501,12 +1511,18 @@ AtPlotterWidget pw;
 			  "AtPlotter has no Y axis defined");
      }
      else if (pp->yaxis && ISDISPLAYED(pp->yaxis)) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 4\n");
+#endif
 	  dummy_min = LIN_MIN;
 	  dummy_max = LIN_MAX;
 	  AtAxisAskRange(pp->yaxis, &dummy_min, &dummy_max);
      }
 
      if (bbp->x2max > bbp->x2min) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 5\n");
+#endif
 	  if (pp->x2axis)
 	       AtAxisAskRange(pp->x2axis, &bbp->x2min, &bbp->x2max);
 	  else
@@ -1514,12 +1530,18 @@ AtPlotterWidget pw;
 			  "AtPlotter has no X2 axis defined");
      }
      else if (pp->x2axis && ISDISPLAYED(pp->x2axis)) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 6\n");
+#endif
 	  dummy_min = LIN_MIN;
 	  dummy_max = LIN_MAX;
 	  AtAxisAskRange(pp->x2axis, &dummy_min, &dummy_max);
      }
 
      if (bbp->y2max > bbp->y2min) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 7\n");
+#endif
 	  if (pp->y2axis)
 	       AtAxisAskRange(pp->y2axis, &bbp->y2min, &bbp->y2max);
 	  else
@@ -1527,6 +1549,9 @@ AtPlotterWidget pw;
 			  "AtPlotter has no Y2 axis defined");
      }
      else if (pp->y2axis && ISDISPLAYED(pp->y2axis)) {
+#ifdef TRACE
+     fprintf(stderr,"Plotter: calling AtAxisAskRange 8\n");
+#endif
 	  dummy_min = LIN_MIN;
 	  dummy_max = LIN_MAX;
 	  AtAxisAskRange(pp->y2axis, &dummy_min, &dummy_max);
@@ -1560,7 +1585,9 @@ Region region;
      AtPlotterPart *pp = &pw->plotter;
      AtPlotterLayout *lp = &pw->plotter.layout;
      int i;
-
+#ifdef TRACE
+     fprintf(stderr,"Plotter: begin Redraw\n");
+#endif
      /* First, the title */
      if (pw->plotter.show_title && pw->plotter.title_text) {
 	  AtTextDraw(XtDisplay(pw), win, drw, pw->plotter.title_gc,
@@ -1596,6 +1623,9 @@ Region region;
 	       AtPlotDraw(CHILD(pw, i), XtDisplay(pw), drw, region, False);
 	  }
      }
+#ifdef TRACE
+     fprintf(stderr,"Plotter: end Redraw\n");
+#endif
 }
 
 /*
@@ -1702,20 +1732,33 @@ Region region;
       * layout_required (probably an axis because something like the
       * number width has changed).
       */
+
 recalc_again:
      pp->in_layout_mode = True;
-
+#ifdef TRACE
+     fprintf(stderr, "rescale required? %d\n", pp->rescale_required);
+#endif
      if (pp->rescale_required)
 	  numbers_moved |= DecideAxisValues(pw);
 
+#ifdef TRACE
+	  fprintf(stderr, "layout_required? %d\n", pp->layout_required);
+	  fprintf(stderr, "recalc legend? %d\n", RecalcLegend(pw));
+#endif
+
      if (pp->layout_required || RecalcLegend(pw))
 	  pixels_moved |= Layout(pw);
 
+
      /*
       * Get the current axis positions and call layout callback list procs
       * to give applications or other plotters the chance to do some axis
       * alignments if this is needed.
       */
+#ifdef TRACE
+	  fprintf(stderr, "has layout? %d\n", HasLayoutCB(pw));
+#endif
+
      if (HasLayoutCB(pw)) {
 	  AtAxisPositions ap;
 
@@ -2005,6 +2048,9 @@ AtPlotterWidget w;
 XtValueMask *vm;
 XSetWindowAttributes *wa;
 {
+#ifdef TRACE
+  fprintf(stderr,"Plotter: start plotter realize\n");
+#endif
      (superclass->core_class.realize)((Widget) w, vm, wa);
      /* Calculate initial boundingbox */
      (void) NewRawBoundingBox(w);
@@ -2014,6 +2060,9 @@ XSetWindowAttributes *wa;
 	  InstallHandlers(w);
 	  XDefineCursor(XtDisplay(w), XtWindow(w), w->plotter.plotter_cursor);
      }
+#ifdef TRACE
+  fprintf(stderr,"Plotter: end plotter realize\n");
+#endif
 }
 
 /*
