@@ -906,6 +906,7 @@ AtText *t;
     int i,j;
     unsigned long foreground;
     unsigned long background;
+    XGCValues values;
     int w,h;  /* width, height */
 
     if (dpy != AtFontFamilyGetDisplay(t->family)) {
@@ -937,8 +938,9 @@ AtText *t;
      * Since everything is unrotated, use w and h backwards
      */
     pixmap = XCreatePixmap(dpy, d, h, w, wa.depth);
-    foreground = gc->values.foreground;
-    background = gc->values.background;
+    XGetGCValues(dpy, gc, GCForeground | GCBackground, &values);
+    foreground = values.foreground;
+    background = values.background;
     XSetForeground(dpy, localgc, background);
     XFillRectangle(dpy, pixmap, localgc, 0, 0, h, w);
     XSetForeground(dpy, localgc, foreground);
