@@ -72,7 +72,7 @@ static XtResource resources[] = {
   {
      XtNaxisTransform, XtCAxisTransform,
      XtRTransform, sizeof(AtTransform),
-     off(axis_transform), XtRTransform, (XtPointer) AtTransformLINEAR
+     off(axis_transform), XtRImmediate, (XtPointer) AtTransformLINEAR
   },
   {
      XtNautoScale, XtCAutoScale,
@@ -817,7 +817,7 @@ XrmValue *args;
 Cardinal num_args;
 XrmValue *from, *to;
 {
-     AtTransform transform;
+static     AtTransform transform;
 
      transform = AtTransformINVALID;
 
