@@ -13,17 +13,16 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Tue Jun 15 04:25:14 MET DST 1993 on dec4.wu-wien.ac.at
+ * Creation: Fri Dec  3 21:45:02 EST 1993 on syllog2.watson.ibm.com
  * Author: genc
- * Version: 0.92
+ * Version: 0.95999999999999996
+
  */
 #ifdef ATHENAR5
-#include <stdio.h>
 
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include "wafe.h"
+#define ATHENAR5_C
 
+#include "wafe.h"                             
 #include <X11/Xaw/Panner.h>
 #include <X11/Xaw/Porthole.h>
 #include <X11/Xaw/Tree.h>
@@ -32,18 +31,18 @@
 extern void portholeCallbackProc();
 extern void pannerCallbackProc();
 
-#define talk(pa, po, ch) XtAddCallback(po, XtNreportCallback, portholeCallbackProc, (XtPointer) pa); XtAddCallback(pa, XtNreportCallback, pannerCallbackProc, (XtPointer) ch)
+#define XawTalk(pa, po, ch) XtAddCallback(po, XtNreportCallback, portholeCallbackProc, (XtPointer) pa); XtAddCallback(pa, XtNreportCallback, pannerCallbackProc, (XtPointer) ch)
 
 /* 
  * void
- * talk
- * 	in: Widget		# panner
- * 	in: Widget		# porthole
+ * XawTalk
+ * 	in: PannerWidget	# panner
+ * 	in: PortholeWidget	# porthole
  * 	in: Widget		# managed widget
  */
 
 int 
-com_talk(clientData, comInterpreter, argc, argv)
+com_XawTalk(clientData, comInterpreter, argc, argv)
 ClientData    clientData;
 Tcl_Interp   *comInterpreter;
 int           argc;
@@ -53,33 +52,33 @@ char        **argv;
      Widget localVar2;
      Widget localVar3;
 
-     DBUG_ENTER("talk");  
+     DBUG_ENTER("XawTalk");  
 
      if (argc != 4) 
 	 {
-	 argcError("talk","",3,argc);
+	 argcError("XawTalk","",3,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
-     if (!(localVar1 = (Widget) name2Widget(argv[1])))
+     if (!(localVar1 =  name2WidgetOfClass(argv[1],pannerWidgetClass,"Panner")))
           {
-          convError("talk","1",argv[1],"Widget");
+          convError("XawTalk","1",argv[1],"PannerWidget");
           DBUG_RETURN (TCL_ERROR);
           }
 
-     if (!(localVar2 = (Widget) name2Widget(argv[2])))
+     if (!(localVar2 =  name2WidgetOfClass(argv[2],portholeWidgetClass,"Porthole")))
           {
-          convError("talk","2",argv[2],"Widget");
+          convError("XawTalk","2",argv[2],"PortholeWidget");
           DBUG_RETURN (TCL_ERROR);
           }
 
      if (!(localVar3 = (Widget) name2Widget(argv[3])))
           {
-          convError("talk","3",argv[3],"Widget");
+          convError("XawTalk","3",argv[3],"Widget");
           DBUG_RETURN (TCL_ERROR);
           }
 
-     talk(localVar1,localVar2,localVar3);
+     XawTalk(localVar1,localVar2,localVar3);
 
      DBUG_RETURN (TCL_OK);
      }     
@@ -88,12 +87,12 @@ char        **argv;
 /* 
  * void
  * XawTreeForceLayout
- * 	in: Widget		# tree Widget
+ * 	in: TreeWidget		# tree Widget
 
  */
 
 int 
-com_treeForceLayout(clientData, comInterpreter, argc, argv)
+com_XawTreeForceLayout(clientData, comInterpreter, argc, argv)
 ClientData    clientData;
 Tcl_Interp   *comInterpreter;
 int           argc;
@@ -101,17 +100,17 @@ char        **argv;
      {
      Widget localVar1;
 
-     DBUG_ENTER("treeForceLayout");  
+     DBUG_ENTER("XawTreeForceLayout");  
 
      if (argc != 2) 
 	 {
-	 argcError("treeForceLayout","",1,argc);
+	 argcError("XawTreeForceLayout","",1,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
-     if (!(localVar1 = (Widget) name2Widget(argv[1])))
+     if (!(localVar1 =  name2WidgetOfClass(argv[1],treeWidgetClass,"Tree")))
           {
-          convError("treeForceLayout","1",argv[1],"Widget");
+          convError("XawTreeForceLayout","1",argv[1],"TreeWidget");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -122,27 +121,27 @@ char        **argv;
 
 
 #else
-#include <string.h>
+#include "wafe.h"
 #endif
 
 void
 Initialize_XawR5Gen()
 {
 #ifdef ATHENAR5
- createWidgetCommand("panner",pannerWidgetClass,NULL);
- createWidgetCommand("porthole",portholeWidgetClass,NULL);
- createWidgetCommand("repeater",repeaterWidgetClass,NULL);
- createWidgetCommand("tree",treeWidgetClass,NULL);
+ createWidgetCommand("Panner",pannerWidgetClass,NULL,False);
+ createWidgetCommand("Porthole",portholeWidgetClass,NULL,False);
+ createWidgetCommand("Repeater",repeaterWidgetClass,NULL,False);
+ createWidgetCommand("Tree",treeWidgetClass,NULL,False);
 #ifdef XAWR5
- strcat(packages,"XAWR5 ");
+ strcat(wafePackages,"XAWR5 ");
 #endif
- Tcl_CreateCommand(interpreter, "talk", com_talk, NULL, NULL);
- Tcl_CreateCommand(interpreter, "treeForceLayout", com_treeForceLayout, NULL, NULL);
+ Tcl_CreateCommand(wafeInterpreter, "XawTalk", com_XawTalk, NULL, NULL);
+ Tcl_CreateCommand(wafeInterpreter, "XawTreeForceLayout", com_XawTreeForceLayout, NULL, NULL);
 
 #else
-createWidgetCommand("panner",NULL,NULL);
-createWidgetCommand("porthole",NULL,NULL);
-createWidgetCommand("repeater",NULL,NULL);
-createWidgetCommand("tree",NULL,NULL);
+ createWidgetCommand("Panner",NULL,NULL,False);
+ createWidgetCommand("Porthole",NULL,NULL,False);
+ createWidgetCommand("Repeater",NULL,NULL,False);
+ createWidgetCommand("Tree",NULL,NULL,False);
 #endif
 }
