@@ -13,28 +13,31 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Tue Jun 15 04:25:51 MET DST 1993 on dec4.wu-wien.ac.at
+ * Creation: Mon Oct 11 06:38:58 MET 1993 on dec1.wu-wien.ac.at
  * Author: genc
- * Version: 0.92
+ * Version: 0.95999999999999996
+
  */
 #ifdef XMGRAPH
-#include <stdio.h>
 
+#define XMGRAPH_C
+
+#include <stdio.h>
 #include <X11/StringDefs.h>
 #include <Xm/Xm.h>
-
-#include "wafe.h"
+#include "wafe.h"                             
+#define ARC(w) (XmArcWidget)(w)
 
 #include <Graph.h>
 #include <Arc.h>
 /* 
  * void
  * XmArcGetPos
- * 	in: Widget		# name of arc Widget
- * 	out: Position		# out: x1
- * 	out: Position		# out: y1
- * 	out: Position		# out: x2
- * 	out: Position		# out: y2
+ * 	in: ARC(XmArcWidget)			# name of arc Widget
+ * 	out: Position				# out: x1
+ * 	out: Position				# out: y1
+ * 	out: Position				# out: x2
+ * 	out: Position				# out: y2
  */
 
 int 
@@ -59,25 +62,25 @@ char        **argv;
 	 DBUG_RETURN (TCL_ERROR);
          }
 
-     if (!(localVar1 = (Widget) name2Widget(argv[1])))
+     if (!(localVar1 =  name2WidgetOfClass(argv[1],xmArcWidgetClass,"XmArc")))
           {
-          convError("mArcGetPos","1",argv[1],"Widget");
+          convError("mArcGetPos","1",argv[1],"XmArcWidget");
           DBUG_RETURN (TCL_ERROR);
           }
 
-     XmArcGetPos(localVar1,&localVar2,&localVar3,&localVar4,&localVar5);
+     XmArcGetPos(ARC(localVar1),&localVar2,&localVar3,&localVar4,&localVar5);
 
      sprintf(conversionBuffer, "%hd", localVar2);
-     Tcl_SetVar(interpreter, argv[2], conversionBuffer, 0);
+     Tcl_SetVar(wafeInterpreter, argv[2], conversionBuffer, 0);
 
      sprintf(conversionBuffer, "%hd", localVar3);
-     Tcl_SetVar(interpreter, argv[3], conversionBuffer, 0);
+     Tcl_SetVar(wafeInterpreter, argv[3], conversionBuffer, 0);
 
      sprintf(conversionBuffer, "%hd", localVar4);
-     Tcl_SetVar(interpreter, argv[4], conversionBuffer, 0);
+     Tcl_SetVar(wafeInterpreter, argv[4], conversionBuffer, 0);
 
      sprintf(conversionBuffer, "%hd", localVar5);
-     Tcl_SetVar(interpreter, argv[5], conversionBuffer, 0);
+     Tcl_SetVar(wafeInterpreter, argv[5], conversionBuffer, 0);
 
      DBUG_RETURN (TCL_OK);
      }     
@@ -85,20 +88,20 @@ char        **argv;
 
 #include <Graph.h>
 #else
-#include <string.h>
+#include "wafe.h"
 #endif
 
 void
 Initialize_XmGraphGen()
 {
 #ifdef XMGRAPH
- createWidgetCommand("mArc",xmArcWidgetClass,NULL);
- createWidgetCommand("mGraph",xmGraphWidgetClass,NULL);
- createWidgetCommand("mScrolledGraph",xmGraphWidgetClass,XmCreateScrolledGraph);
+ createWidgetCommand("mArc",xmArcWidgetClass,NULL,False);
+ createWidgetCommand("mGraph",xmGraphWidgetClass,NULL,False);
+ createWidgetCommand("mScrolledGraph",xmGraphWidgetClass,XmCreateScrolledGraph,False);
 #ifdef XMGRAPH
- strcat(packages,"XMGRAPH ");
+ strcat(wafePackages,"XMGRAPH ");
 #endif
- Tcl_CreateCommand(interpreter, "mArcGetPos", com_mArcGetPos, NULL, NULL);
+ Tcl_CreateCommand(wafeInterpreter, "mArcGetPos", com_mArcGetPos, NULL, NULL);
 
 #else
 #endif
