@@ -15,12 +15,12 @@
  * implied warranty.
  *
  * Date: Mon, Apr 13 1992
- * Author: Stefan Nusser
+ * Author: Stefan Nusser, Gustaf Neumann
  * Version: 0.9
  */
 
 /*
- *             ACTIONSANDCALLBACKS.C
+ *             ACTIONSCB.C
  *
  *             This file contains all commands dealing with actions,
  *             callbacks, accelerators, etc and all related functions:
@@ -31,16 +31,12 @@
  *             execCallbackProc
  */
 
+#define ACTIONSCB_C
 
-#include <stdio.h>
-#include <sys/types.h>
-
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
+#include "wafe.h"
 
 #ifdef ATHENA
 #include<X11/Xaw/AsciiText.h>
-
 #ifdef XEDW
 #include "Xedw/XedwList.h"
 #endif
@@ -51,7 +47,7 @@
 #include <X11/Xaw/Porthole.h>
 #endif
 
-#include "wafe.h"                             
+
 
 /*
  * Additionally, we need the following handler procedures, which are located in the
@@ -92,7 +88,7 @@ Cardinal  *argc;
 
     DBUG_ENTER("execActionProc");
 
-    strcpy(assembled, "");
+    assembled[0] = '\0';
 
     for (index = 0; index < *argc; index++)
 	{
@@ -100,8 +96,7 @@ Cardinal  *argc;
 	strcat(assembled, " ");
 	}
 
-    DBUG_PRINT("action",
-	       ("Assembled command: %s", assembled));
+    DBUG_PRINT("action", ("Assembled command: %s", assembled));
 
     while (*input != '\0')
 	{
@@ -237,12 +232,8 @@ Cardinal  *argc;
 
     *output = '\0';
 
-    DBUG_PRINT("action",
-	       ("command:<<%s>>", command));
-
-    if(Tcl_Eval(interpreter, command, 0, NULL) == TCL_ERROR)
-	fprintf(stderr, "Wafe(execActionProc): %s\n", interpreter->result);
-
+    DBUG_PRINT("action", ("command:<<%s>>", command));
+    (void) wafeEval(wafeInterpreter, command, "execActionProc");
     DBUG_VOID_RETURN;
     }
 
@@ -263,7 +254,7 @@ XtPointer  clientData, callData;
                    XtNy, &y,
                    NULL);
 
-     XtTranslateCoords(topLevel, x, y, &xRoot, &yRoot);
+     XtTranslateCoords(wafeTopLevel, x, y, &xRoot, &yRoot);
      XtVaSetValues(w,
                          XtNx, xRoot+(wao->xoffset),
                          XtNy, yRoot+(wao->yoffset),
@@ -278,14 +269,13 @@ Widget     w;
 XtPointer  clientData, callData;
      {
      Window root, child;
-     int  x, y, win_x,win_y,width,height, screenWidth, screenHeight;
-     Position xRoot,yRoot,wx,wy;
+     int  x, y, win_x,win_y;
      unsigned int mask;
      int  offset = (int)clientData;
 
      DBUG_ENTER("positionCursorCallbackProc");
 
-     (void) XQueryPointer(XtDisplay(w), XtWindow(topLevel),
+     (void) XQueryPointer(XtDisplay(w), XtWindow(wafeTopLevel),
                          &root, &child,
                          &x, &y, &win_x, &win_y,
                          &mask);
@@ -337,32 +327,8 @@ char           *cbAttribute;
 XtCallbackProc  proc;
 char           * arg;
     {   
-    Widget           subpart, pop;
+    Widget           pop;
 
-
-    if (proc == execCallbackProc)   /* Care for callData passing... */
-          {
-#ifdef NEVER
-#ifdef ATHENA
-#ifdef XEDW
-          if ((XtClass(w) == xedwListWidgetClass) && (!strcmp(cbAttribute, "callback")))
-              XtAddCallback(w, cbAttribute, edwListCallbackProc, arg);
-          else
-#endif
-	      if (XtClass(w) == asciiTextWidgetClass)
-		  {
-		  XtVaGetValues(w, XtNtextSource, &subpart, NULL);
-		  XtAddCallback(subpart, cbAttribute, execCallbackProc, arg);
-		  }
-	      else
-#endif
-		  XtAddCallback(w, cbAttribute, execCallbackProc, arg);
-#else
-	  fprintf(stderr,"obsolete call! use \"sV %s callback %s\" instead!\n",
-		  XtName(w),arg);
-#endif
-	  }
-    else
     if (proc == positionCursorCallbackProc)   /* Additional argument is offset... */
 	{
 	int offset;
@@ -384,10 +350,10 @@ char           * arg;
 	char *colon, *slash;
 
 	wao = (widgetAndOffset *)XtMalloc(sizeof(widgetAndOffset));
-	if (colon = (char*)rindex(arg,':'))
+	if ((colon = (char*)rindex(arg,':')))
 	    {
 	    *colon = '\0'; colon ++;
-	    if (slash = (char*)rindex(colon,'/'))
+	    if ((slash = (char*)rindex(colon,'/')))
 		{
 		*slash = '\0'; slash ++;
 		if (!sscanf(slash, "%d", &(wao->yoffset))) 
