@@ -94,7 +94,7 @@ rddStartAction (w, event, args, nargs)
 	Widget w;
 	XButtonEvent *event;
 	String *args;
-	Cardinal *nargs;
+	int *nargs;
 {
 	dragflag = 0;
 	bx = event->x;
@@ -110,7 +110,7 @@ rddDragAction (w, event, args, nargs)
 	Widget w;
 	XButtonEvent *event;
 	String *args;
-	Cardinal *nargs;
+	int *nargs;
 {
 	DEBUG (fprintf(stderr, "rddDragAction\n"););
 	if (dragflag == 0 && ((abs(bx - event->x)) > 10 ||
@@ -159,7 +159,7 @@ rddDropAction (w, event, args, nargs)
 	Widget w;
 	XButtonEvent *event;
 	String *args;
-	Cardinal *nargs;
+	int *nargs;
 {
 	DEBUG (fprintf(stderr, "rddDropAction\n"););
 	if (dragflag == 1)
@@ -253,10 +253,11 @@ rddInit(shell, context)
 	XtAppContext context;
 {
 	/* Should NOT need to CAST here!*/
-	static XtActionsRec actions[3] = {
-			"rddStartAction",   (XtActionProc)rddStartAction,
-			"rddDragAction",    (XtActionProc)rddDragAction,
-			"rddDropAction",    (XtActionProc)rddDropAction,
+	static XtActionsRec actions[] = {
+			{"rddStartAction",   (XtActionProc)rddStartAction},
+			{"rddDragAction",    (XtActionProc)rddDragAction},
+			{"rddDropAction",    (XtActionProc)rddDropAction},
+			{NULL,NULL}
 	};
 
 	DEBUG (fprintf(stderr, "rddInit entered\n"););
@@ -391,7 +392,7 @@ rddGetSelection (data)
 {
 	int n;
 	Atom type;
-	int format, count;
+	int format;
 	u_long leftover;
 
 	DEBUG (fprintf (stderr, "rddGetSelection entered\n"););
@@ -648,7 +649,6 @@ void rddAppMainLoop(app)
 	XtAppContext app;
 {
     XEvent event;
-	Widget w;
 
     for (;;)
 	{
