@@ -18,24 +18,16 @@
  * Author: Stefan Nusser
  * Version: 0.9
  */
-
-#include <stdio.h>
-
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/Xatom.h>
+#define SELECTION_C
 
 #include "wafe.h"
+#include <X11/Xatom.h>
 
 char *selected = NULL;
 char *loseSelectionCommand = NULL;
 char *doneSelectionCommand = NULL;
 
-char     one[] = "1";
-char     zero[] = "0";
-char     empty[] = "";
-
-Boolean
+static Boolean
 convertProc(owner, selection, target, typeReturn, valueReturn, 
             lengthReturn, formatReturn)
 Widget          owner;
@@ -50,7 +42,7 @@ int            *formatReturn;
 
     if (*target != XA_STRING)
 	{
-	fprintf(stderr, "Requested target-type %d is unsupported!\n",*target);
+	fprintf(stderr, "Requested target-type %ld is unsupported!\n",*target);
 	DBUG_RETURN(False);
 	}
     else
@@ -63,32 +55,24 @@ int            *formatReturn;
 	}
     }
            
-void
+static void
 loseSelectionProc(w, selection)
 Widget     w;
 Atom      *selection;
      {
      DBUG_ENTER("loseSelectionProc");
-
-     DBUG_PRINT("selection",("Executing %s", loseSelectionCommand));
-     if (Tcl_Eval(interpreter, loseSelectionCommand, 0, NULL) == TCL_ERROR)
-          fprintf(stderr, "loseSelectionProc: Error in executing command\n");
-          
+     (void) wafeEval(wafeInterpreter, loseSelectionCommand, "loseSelectionProc");
      DBUG_VOID_RETURN;
      }
 
-void
+static void
 doneSelectionProc(w, selection, target)
 Widget     w;
 Atom      *selection;
 Atom      *target;
     {
-    DBUG_ENTER("doenSelectionDoneProc");
-    
-    DBUG_PRINT("selection",("Executing %s", doneSelectionCommand));
-    if (Tcl_Eval(interpreter, doneSelectionCommand, 0, NULL) == TCL_ERROR)
-	fprintf(stderr, "doneSelectionProc: Error in executing command\n");
-          
+    DBUG_ENTER("doneSelectionDoneProc");
+    (void) wafeEval(wafeInterpreter, doneSelectionCommand,"doneSelectionProc");
     DBUG_VOID_RETURN;
     }
 
@@ -108,14 +92,12 @@ int           *format;
     DBUG_ENTER("getSelectionValueProc");
 
     if (value == NULL)
-	primary = empty;
+	primary = "";
     else
 	primary = (char *)value;
     
-    Tcl_SetVar(interpreter, "PRIMARY", primary, TCL_GLOBAL_ONLY);
-
-    if (Tcl_Eval(interpreter, command, 0, NULL) == TCL_ERROR)
-	fprintf(stderr, "%s\n", interpreter->result);
+    Tcl_SetVar(wafeInterpreter, "PRIMARY", primary, TCL_GLOBAL_ONLY);
+    (void) wafeEval(wafeInterpreter, command, "selectionProc");
 
     DBUG_VOID_RETURN;
     }
@@ -127,7 +109,6 @@ String      string;
 String      loseSelection;
 String      doneSelection;
     {
-    int   i;
     XtLoseSelectionProc    loseProc = NULL;
     XtSelectionDoneProc   doneProc = NULL;
 
@@ -154,7 +135,7 @@ String      doneSelection;
 	}
 
 
-    if (XtOwnSelection(topLevel, XA_PRIMARY,  CurrentTime,
+    if (XtOwnSelection(wafeTopLevel, XA_PRIMARY,  CurrentTime,
 		       convertProc, loseProc, doneProc))
 	return True;
     else  
